/*  Copyright © 2016 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 12/03/16.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers   */
#include "bits/mc-splitfunc-approx.h"



namespace duct {
  
  namespace __details {
    
    void splitting_functions_approx<fin, ordering::angular>::bind(double v)
    {
      using _Split = __details::splitting_functions_base<fin>;
      
      /*   binding the z-mapping in the low z region  */
      _M_zmapL.bindL(_Base::_M_pars, v);
      
      
      /*    setting up the soft spline  (low z region)  */
      _M_softL.assign(unitweight{}, [=](double w) -> double {
        auto z = _M_zmapL.w2z(splitting_type::soft, w);
        auto y = ordering_traits<fin,ordering_type>::main::mapping_to_y(_Base::_M_pars, v, z.first);
        
        return y.second*z.second*_Split::parton_gluon(_Base::_M_pars, y.first, z.first);
      });
      
      
      /*    setting up the collinear spline  (low z region)  */
      _M_collL.assign(unitweight{}, [=](double w) -> double {
        auto z = _M_zmapL.w2z(splitting_type::collinear, w);
        auto y = ordering_traits<fin,ordering_type>::main::mapping_to_y(_Base::_M_pars, v, z.first);
        
        return y.second*z.second*(_Base::_M_pars->fl
                                  ?  _Split::quark_gluon(_Base::_M_pars, y.first, z.first)
                                  : (_Split::gluon_gluon(_Base::_M_pars, y.first, z.first)
                                     + _Split::quark_antiquark(_Base::_M_pars, y.first, z.first)));
      });

    
    
      /*   binding the z-mapping in the low z region  */
      _M_zmapH.bindH(_Base::_M_pars, v);
      
      
      /*    setting up the soft spline  (low z region)  */
      _M_softH.assign(unitweight{}, [=](double w) -> double {
        auto z = _M_zmapH.w2z(splitting_type::soft, w);
        auto y = ordering_traits<fin,ordering_type>::main::mapping_to_y(_Base::_M_pars, v, z.first);
        
        return y.second*z.second*_Split::parton_gluon(_Base::_M_pars, y.first, z.first);
      });
      
      
      /*    setting up the collinear spline  (low z region)  */
      _M_collH.assign(unitweight{}, [=](double w) -> double {
        auto z = _M_zmapH.w2z(splitting_type::collinear, w);
        auto y = ordering_traits<fin,ordering_type>::main::mapping_to_y(_Base::_M_pars, v, z.first);
        
        return y.second*z.second*(_Base::_M_pars->fl
                                  ?  _Split::quark_gluon(_Base::_M_pars, y.first, z.first)
                                  : (_Split::gluon_gluon(_Base::_M_pars, y.first, z.first)
                                     + _Split::quark_antiquark(_Base::_M_pars, y.first, z.first)));
      });
    }
    
    
    std::pair<double,double>
    splitting_functions_approx<fin, ordering::angular>::mappingL_chi2z(splitting_type idx, double chi) const
    {
      /*   chi ==> w using the splines    */
      double w, jac;
      
      switch(idx) {
        case splitting_type::soft:
          w = _M_softL.integral_inverse(chi*_M_softL.integral());
          jac = _M_softL.integral()/_M_softL(w);
          break;
          
        case splitting_type::collinear:
          w = _M_collL.integral_inverse(chi*_M_collL.integral());
          jac = _M_collL.integral()/_M_collL(w);
          break;
          
        default:
          return {0.5, 0.0};
      }
      
      /*   from w ==> z  using the pdf integrals    */
      auto z = _M_zmapL.w2z(idx, w);
      z.second *= jac;
      
      return z;
    }
    
    
    std::pair<double,double>
    splitting_functions_approx<fin, ordering::angular>::mappingH_chi2z(splitting_type idx, double chi) const
    {
      /*   chi ==> w using the splines    */
      double w, jac;
      
      switch(idx) {
        case splitting_type::soft:
          w = _M_softH.integral_inverse(chi*_M_softH.integral());
          jac = _M_softH.integral()/_M_softH(w);
          break;
          
        case splitting_type::collinear:
          w = _M_collH.integral_inverse(chi*_M_collH.integral());
          jac = _M_collH.integral()/_M_collH(w);
          break;
          
        default:
          return {0.5, 0.0};
      }
      
      /*   from w ==> z  using the pdf integrals    */
      auto z = _M_zmapH.w2z(idx, w);
      z.second *= jac;
      
      return z;
    }
    
  }   //   namespace __details
}
