/*  Copyright © 2015 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 10/12/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __math_zeta_h__
#define __math_zeta_h__ 1


/*   std includes   */
#include <cstddef>

/*   project headers   */


namespace duct {
 
  namespace __details {
  
    /*   we put the zeta values into tables   */
    extern const double zetam1_pos_int_table[101];
    extern const double zeta_neg_int_table[50];
    
    template<std::size_t N, bool is_positive = (N >= 0)>
    struct zeta_helper;
    
    template<std::size_t N>
    struct zeta_helper<N, true>
    {
      static constexpr double value = (N > 100 ? 1.0 : 1.0 + __details::zetam1_pos_int_table[N]);
    };
   
    template<std::size_t N>
    struct zeta_helper<N, false>
    {
      static constexpr double value = (!(N % 2) || N < -100 ? 0.0 : __details::zeta_neg_int_table[-(N+1)/2]);
    };
    
  }   /*   namespace __details   */
  
  
  /*  We put the static version of math function on namespace meta.
   *  This way we can implement their runtime version too.
   */
  namespace meta {
    
    template<std::size_t N>
    double zeta = __details::zeta_helper<N>::value;
 
  }   /*   namespace meta   */
  
  
  
  /*   Riemann zeta at integer values   */
  inline double zeta(int n)
  {
    if(n < 0)  return (!(n % 2) || n < -100 ? 0.0 : __details::zeta_neg_int_table[-(n+1)/2]);
    else if(n <= 100) return 1.0 + __details::zetam1_pos_int_table[n];
    else return 1.0;
  }
  

}

#endif



