/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 9/8/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std headers   */
#include <iostream>

/*   C headers   */
#include <getopt.h>



extern void main_run_help();
extern void main_res_help();
extern void main_mod_help();

extern int main_mod(char, int, char **);
extern int main_run(int, char **);
extern int main_res(int, char **);
extern int main_pdf(int, char **);




int main(int argc, char **argv)
{
  ::option longopt[] = {
    /*    making & managing modules   */
    {"new",    required_argument, nullptr, 0},
    {"add-to", required_argument, nullptr, 0},
    {"build",  required_argument, nullptr, 0},
    {"clone",  required_argument, nullptr, 0},
    {"module", required_argument, nullptr, 0},
   
    /*   run the module and generate the results   */
    {"run",    required_argument, nullptr, 0},
    {"result", required_argument, nullptr, 0},
    
    /*   precalculates the PDF tabels    */
    {"init-pdf", optional_argument, nullptr, 0},

    /*   help   */
    {"help",   no_argument,       nullptr, 'h'},
    {nullptr, 0, nullptr, 0}
  };
  
  try {
    while(1) {
      int optidx = 0;
      int c = ::getopt_long(argc, argv, "h", longopt, &optidx);
      if(c == -1) break;
      
      switch(c) {
        case 'h':
          main_mod_help();
          main_run_help();
          main_res_help();
          exit(0);
          break;
        case 0:
          switch(optidx) {
            case 0: main_mod('n', argc, argv); break;
            case 1: main_mod('a', argc, argv); break;
            case 2: main_mod('b', argc, argv); break;
            case 3: main_mod('c', argc, argv); break;
            case 4: main_mod('m', argc, argv); break;
           
            case 5: main_run(argc, argv); break;
            case 6: main_res(argc, argv); break;
            case 7: main_pdf(argc, argv); break;
          }
          break;
        case '?': break;
        default: throw; break;
      }
    }
  } catch(const char *message) {
    std::cerr<<"Error : "<<message<<std::endl;
    return 1;
  } catch(const std::exception& e) {
    std::cerr<<"Error : "<<e.what()<<std::endl;
    return 1;
  } catch(...) {
    std::cerr<<"Unexpected error!"<<std::endl;
    return 1;
  }
  
  return 0;
}
